@@ -2,6 +2,8 @@ var searchData=
 [
   ['libinput_5fbutton_5fstate',['libinput_button_state',['../group__device.html#ga543a8bb3f2d7421f76e63bb4ee2fb5bf',1,'libinput.h']]],
   ['libinput_5fconfig_5fclick_5fmethod',['libinput_config_click_method',['../group__config.html#ga3e5dc19a83752430d5e8be7fb2020663',1,'libinput.h']]],
+  ['libinput_5fconfig_5fdrag_5flock_5fstate',['libinput_config_drag_lock_state',['../group__config.html#gaedaabeb8e33f58a032c2e099b91f4579',1,'libinput.h']]],
+  ['libinput_5fconfig_5fmiddle_5femulation_5fstate',['libinput_config_middle_emulation_state',['../group__config.html#ga43d9b6271fdbf41b3a0591dea9d1b86a',1,'libinput.h']]],
   ['libinput_5fconfig_5fscroll_5fmethod',['libinput_config_scroll_method',['../group__config.html#ga6466e6b9c55e6835a05c7c29c8b7a844',1,'libinput.h']]],
   ['libinput_5fconfig_5fsend_5fevents_5fmode',['libinput_config_send_events_mode',['../libinput_8h.html#a56c9886b0161ecdb9b351d8dedc212e4',1,'libinput.h']]],
   ['libinput_5fconfig_5fstatus',['libinput_config_status',['../group__config.html#ga33ff306ed8505129adce4c092a3ed175',1,'libinput.h']]],
@@ -10,7 +12,7 @@ var searchData=
   ['libinput_5fevent_5ftype',['libinput_event_type',['../group__base.html#gac30276a06e8b1434b959f2c8dde74f7c',1,'libinput.h']]],
   ['libinput_5fkey_5fstate',['libinput_key_state',['../group__device.html#ga360bdea89a75606472a0b5d72678bd17',1,'libinput.h']]],
   ['libinput_5fled',['libinput_led',['../group__device.html#ga37ec9afaec2e5407e3c5248faedd1970',1,'libinput.h']]],
-  ['libinput_5flog_5fpriority',['libinput_log_priority',['../libinput_8h.html#aa7e9ebbd1a13fe2ed6f44e69df3216e1',1,'libinput.h']]],
+  ['libinput_5flog_5fpriority',['libinput_log_priority',['../group__base.html#gaa7e9ebbd1a13fe2ed6f44e69df3216e1',1,'libinput.h']]],
   ['libinput_5fpointer_5faxis',['libinput_pointer_axis',['../group__device.html#ga82cd15b9476a4a60f25470df8d67483c',1,'libinput.h']]],
   ['libinput_5fpointer_5faxis_5fsource',['libinput_pointer_axis_source',['../group__device.html#ga76c012d8f6d7656fb795dc7bdf9d6551',1,'libinput.h']]]
 ];
