@@ -17,6 +17,10 @@ var searchData=
   ['libinput_5fdevice_5fconfig_5fleft_5fhanded_5fget_5fdefault',['libinput_device_config_left_handed_get_default',['../group__config.html#gaa438a14e32894791e435cb43826ef3a9',1,'libinput.h']]],
   ['libinput_5fdevice_5fconfig_5fleft_5fhanded_5fis_5favailable',['libinput_device_config_left_handed_is_available',['../group__config.html#ga755352f2bbb869446aa591dd9d9c5eaf',1,'libinput.h']]],
   ['libinput_5fdevice_5fconfig_5fleft_5fhanded_5fset',['libinput_device_config_left_handed_set',['../group__config.html#ga9bbdef04c07804ce7c121133c04d4edf',1,'libinput.h']]],
+  ['libinput_5fdevice_5fconfig_5fmiddle_5femulation_5fget_5fdefault_5fenabled',['libinput_device_config_middle_emulation_get_default_enabled',['../group__config.html#ga602bf0edd3553972cd2d1f841401578b',1,'libinput.h']]],
+  ['libinput_5fdevice_5fconfig_5fmiddle_5femulation_5fget_5fenabled',['libinput_device_config_middle_emulation_get_enabled',['../group__config.html#ga4669a9e2eeda7ddd648d340935c40ada',1,'libinput.h']]],
+  ['libinput_5fdevice_5fconfig_5fmiddle_5femulation_5fis_5favailable',['libinput_device_config_middle_emulation_is_available',['../group__config.html#gad9d32f1fd776fa112e5eaac6419de6b7',1,'libinput.h']]],
+  ['libinput_5fdevice_5fconfig_5fmiddle_5femulation_5fset_5fenabled',['libinput_device_config_middle_emulation_set_enabled',['../group__config.html#ga584d862a116fb0a26cc95304f8c13eb1',1,'libinput.h']]],
   ['libinput_5fdevice_5fconfig_5fscroll_5fget_5fbutton',['libinput_device_config_scroll_get_button',['../group__config.html#ga83623da437d0114b8e3ac29ab771d799',1,'libinput.h']]],
   ['libinput_5fdevice_5fconfig_5fscroll_5fget_5fdefault_5fbutton',['libinput_device_config_scroll_get_default_button',['../group__config.html#gad4aacefb10fab1e401e211ee9ce0c1e6',1,'libinput.h']]],
   ['libinput_5fdevice_5fconfig_5fscroll_5fget_5fdefault_5fmethod',['libinput_device_config_scroll_get_default_method',['../group__config.html#ga844ea06b952aaee8e0643d85d351afb6',1,'libinput.h']]],
@@ -32,9 +36,12 @@ var searchData=
   ['libinput_5fdevice_5fconfig_5fsend_5fevents_5fget_5fmode',['libinput_device_config_send_events_get_mode',['../group__config.html#gae945f2120764570f0b7c48b69a035543',1,'libinput.h']]],
   ['libinput_5fdevice_5fconfig_5fsend_5fevents_5fget_5fmodes',['libinput_device_config_send_events_get_modes',['../group__config.html#gaa03e5c609b0b199402048fcd2cf85b24',1,'libinput.h']]],
   ['libinput_5fdevice_5fconfig_5fsend_5fevents_5fset_5fmode',['libinput_device_config_send_events_set_mode',['../group__config.html#gaacc79635146b888ddaf1cb35363b22ce',1,'libinput.h']]],
+  ['libinput_5fdevice_5fconfig_5ftap_5fget_5fdefault_5fdrag_5flock_5fenabled',['libinput_device_config_tap_get_default_drag_lock_enabled',['../group__config.html#gaa9ac23a944aa2232397bcd856cc55055',1,'libinput.h']]],
   ['libinput_5fdevice_5fconfig_5ftap_5fget_5fdefault_5fenabled',['libinput_device_config_tap_get_default_enabled',['../group__config.html#gaf6906b7344ee8d446594de08b4d065be',1,'libinput.h']]],
+  ['libinput_5fdevice_5fconfig_5ftap_5fget_5fdrag_5flock_5fenabled',['libinput_device_config_tap_get_drag_lock_enabled',['../group__config.html#ga096a8960db3d6e43c419180270755584',1,'libinput.h']]],
   ['libinput_5fdevice_5fconfig_5ftap_5fget_5fenabled',['libinput_device_config_tap_get_enabled',['../group__config.html#ga8bfe1d541dcadaedeedcd7a98d5b7e07',1,'libinput.h']]],
   ['libinput_5fdevice_5fconfig_5ftap_5fget_5ffinger_5fcount',['libinput_device_config_tap_get_finger_count',['../group__config.html#ga13973d6bd75a99708723e3c6684517c1',1,'libinput.h']]],
+  ['libinput_5fdevice_5fconfig_5ftap_5fset_5fdrag_5flock_5fenabled',['libinput_device_config_tap_set_drag_lock_enabled',['../group__config.html#ga8b72d649a4e05a455c1d0ec4194cb755',1,'libinput.h']]],
   ['libinput_5fdevice_5fconfig_5ftap_5fset_5fenabled',['libinput_device_config_tap_set_enabled',['../group__config.html#ga3caf860883d4e2e42b7842599da03e56',1,'libinput.h']]],
   ['libinput_5fdevice_5fget_5fcontext',['libinput_device_get_context',['../group__device.html#ga90c6868911d8aba103278c9208d5205e',1,'libinput.h']]],
   ['libinput_5fdevice_5fget_5fdevice_5fgroup',['libinput_device_get_device_group',['../group__device.html#ga0edc58e5a022b48a8d8416f63d2860df',1,'libinput.h']]],
@@ -52,6 +59,7 @@ var searchData=
   ['libinput_5fdevice_5fgroup_5fset_5fuser_5fdata',['libinput_device_group_set_user_data',['../group__device.html#gaf252f7535939d813ebe6eb7b2b792446',1,'libinput.h']]],
   ['libinput_5fdevice_5fgroup_5funref',['libinput_device_group_unref',['../group__device.html#ga371124d3d3bec0a7f946acd1c4417efe',1,'libinput.h']]],
   ['libinput_5fdevice_5fhas_5fcapability',['libinput_device_has_capability',['../group__device.html#gaa32d887454e8baf8aaeeb635329aef89',1,'libinput.h']]],
+  ['libinput_5fdevice_5fkeyboard_5fhas_5fkey',['libinput_device_keyboard_has_key',['../group__device.html#ga80efc6bbab881c6785f0184ddc97c5ce',1,'libinput.h']]],
   ['libinput_5fdevice_5fled_5fupdate',['libinput_device_led_update',['../group__device.html#gafdb3ddfb91e6bd38b6356a5bb51547ad',1,'libinput.h']]],
   ['libinput_5fdevice_5fpointer_5fhas_5fbutton',['libinput_device_pointer_has_button',['../group__device.html#gaaf63f10e5120daf174bd7943169f45bf',1,'libinput.h']]],
   ['libinput_5fdevice_5fref',['libinput_device_ref',['../group__device.html#ga3335d55a50f0c2fb0c5f496a3105d467',1,'libinput.h']]],
