@@ -1,8 +1,8 @@
 var indexSectionsWithContent =
 {
-  0: "acdiklnoprst",
+  0: "acdfiklnoprst",
   1: "l",
-  2: "acdlnprst",
+  2: "acdflnprst",
   3: "l",
   4: "co",
   5: "l",
@@ -10,7 +10,7 @@ var indexSectionsWithContent =
   7: "l",
   8: "l",
   9: "adikpt",
-  10: "aclnpst"
+  10: "acflnpst"
 };
 
 var indexSectionNames =
