@@ -1,25 +1,28 @@
 /*
- * Copyright © 2014 Red Hat, Inc.
+ * Copyright © 2014-2015 Red Hat, Inc.
  *
- * Permission to use, copy, modify, distribute, and sell this software and
- * its documentation for any purpose is hereby granted without fee, provided
- * that the above copyright notice appear in all copies and that both that
- * copyright notice and this permission notice appear in supporting
- * documentation, and that the name of the copyright holders not be used in
- * advertising or publicity pertaining to distribution of the software
- * without specific, written prior permission.  The copyright holders make
- * no representations about the suitability of this software for any
- * purpose.  It is provided "as is" without express or implied warranty.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
- * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
- * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
- * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
- * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
- * CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
- * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ * The above copyright notice and this permission notice (including the next
+ * paragraph) shall be included in all copies or substantial portions of the
+ * Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
+#include "config.h"
+
 #include <errno.h>
 #include <limits.h>
 #include <math.h>
@@ -29,7 +32,6 @@
 
 #include "evdev-mt-touchpad.h"
 
-#define DEFAULT_BUTTON_MOTION_THRESHOLD 0.02 /* 2% of size */
 #define DEFAULT_BUTTON_ENTER_TIMEOUT 100 /* ms */
 #define DEFAULT_BUTTON_LEAVE_TIMEOUT 300 /* ms */
 
@@ -144,12 +146,15 @@ tp_button_set_leave_timer(struct tp_dispatch *tp, struct tp_touch *t)
  * as described in the state machine diagram.
  */
 static void
-tp_button_set_state(struct tp_dispatch *tp, struct tp_touch *t,
-		    enum button_state new_state, enum button_event event)
+tp_button_set_state(struct tp_dispatch *tp,
+		    struct tp_touch *t,
+		    enum button_state new_state,
+		    enum button_event event)
 {
 	libinput_timer_cancel(&t->button.timer);
 
 	t->button.state = new_state;
+
 	switch (t->button.state) {
 	case BUTTON_STATE_NONE:
 		t->button.curr = 0;
@@ -235,7 +240,9 @@ tp_button_bottom_handle_event(struct tp_dispatch *tp,
 	case BUTTON_EVENT_IN_BOTTOM_R:
 	case BUTTON_EVENT_IN_BOTTOM_L:
 		if (event != t->button.curr)
-			tp_button_set_state(tp, t, BUTTON_STATE_BOTTOM,
+			tp_button_set_state(tp,
+					    t,
+					    BUTTON_STATE_BOTTOM,
 					    event);
 		break;
 	case BUTTON_EVENT_IN_TOP_R:
@@ -256,8 +263,8 @@ tp_button_bottom_handle_event(struct tp_dispatch *tp,
 
 static void
 tp_button_top_handle_event(struct tp_dispatch *tp,
-			    struct tp_touch *t,
-			    enum button_event event)
+			   struct tp_touch *t,
+			   enum button_event event)
 {
 	switch (event) {
 	case BUTTON_EVENT_IN_BOTTOM_R:
@@ -268,7 +275,9 @@ tp_button_top_handle_event(struct tp_dispatch *tp,
 	case BUTTON_EVENT_IN_TOP_M:
 	case BUTTON_EVENT_IN_TOP_L:
 		if (event != t->button.curr)
-			tp_button_set_state(tp, t, BUTTON_STATE_TOP_NEW,
+			tp_button_set_state(tp,
+					    t,
+					    BUTTON_STATE_TOP_NEW,
 					    event);
 		break;
 	case BUTTON_EVENT_IN_AREA:
@@ -286,8 +295,8 @@ tp_button_top_handle_event(struct tp_dispatch *tp,
 
 static void
 tp_button_top_new_handle_event(struct tp_dispatch *tp,
-				struct tp_touch *t,
-				enum button_event event)
+			       struct tp_touch *t,
+			       enum button_event event)
 {
 	switch(event) {
 	case BUTTON_EVENT_IN_BOTTOM_R:
@@ -298,7 +307,9 @@ tp_button_top_new_handle_event(struct tp_dispatch *tp,
 	case BUTTON_EVENT_IN_TOP_M:
 	case BUTTON_EVENT_IN_TOP_L:
 		if (event != t->button.curr)
-			tp_button_set_state(tp, t, BUTTON_STATE_TOP_NEW,
+			tp_button_set_state(tp,
+					    t,
+					    BUTTON_STATE_TOP_NEW,
 					    event);
 		break;
 	case BUTTON_EVENT_IN_AREA:
@@ -320,18 +331,22 @@ tp_button_top_new_handle_event(struct tp_dispatch *tp,
 
 static void
 tp_button_top_to_ignore_handle_event(struct tp_dispatch *tp,
-				      struct tp_touch *t,
-				      enum button_event event)
+				     struct tp_touch *t,
+				     enum button_event event)
 {
 	switch(event) {
 	case BUTTON_EVENT_IN_TOP_R:
 	case BUTTON_EVENT_IN_TOP_M:
 	case BUTTON_EVENT_IN_TOP_L:
 		if (event == t->button.curr)
-			tp_button_set_state(tp, t, BUTTON_STATE_TOP,
+			tp_button_set_state(tp,
+					    t,
+					    BUTTON_STATE_TOP,
 					    event);
 		else
-			tp_button_set_state(tp, t, BUTTON_STATE_TOP_NEW,
+			tp_button_set_state(tp,
+					    t,
+					    BUTTON_STATE_TOP_NEW,
 					    event);
 		break;
 	case BUTTON_EVENT_IN_BOTTOM_R:
@@ -352,8 +367,8 @@ tp_button_top_to_ignore_handle_event(struct tp_dispatch *tp,
 
 static void
 tp_button_ignore_handle_event(struct tp_dispatch *tp,
-			    struct tp_touch *t,
-			    enum button_event event)
+			      struct tp_touch *t,
+			      enum button_event event)
 {
 	switch (event) {
 	case BUTTON_EVENT_IN_BOTTOM_R:
@@ -379,7 +394,7 @@ tp_button_handle_event(struct tp_dispatch *tp,
 		       enum button_event event,
 		       uint64_t time)
 {
-	struct libinput *libinput = tp->device->base.seat->libinput;
+	struct libinput *libinput = tp_libinput_context(tp);
 	enum button_state current = t->button.state;
 
 	switch(t->button.state) {
@@ -426,18 +441,22 @@ tp_button_handle_state(struct tp_dispatch *tp, uint64_t time)
 		if (t->state == TOUCH_END) {
 			tp_button_handle_event(tp, t, BUTTON_EVENT_UP, time);
 		} else if (t->dirty) {
+			enum button_event event;
+
 			if (is_inside_bottom_right_area(tp, t))
-				tp_button_handle_event(tp, t, BUTTON_EVENT_IN_BOTTOM_R, time);
+				event = BUTTON_EVENT_IN_BOTTOM_R;
 			else if (is_inside_bottom_left_area(tp, t))
-				tp_button_handle_event(tp, t, BUTTON_EVENT_IN_BOTTOM_L, time);
+				event = BUTTON_EVENT_IN_BOTTOM_L;
 			else if (is_inside_top_right_area(tp, t))
-				tp_button_handle_event(tp, t, BUTTON_EVENT_IN_TOP_R, time);
+				event = BUTTON_EVENT_IN_TOP_R;
 			else if (is_inside_top_middle_area(tp, t))
-				tp_button_handle_event(tp, t, BUTTON_EVENT_IN_TOP_M, time);
+				event = BUTTON_EVENT_IN_TOP_M;
 			else if (is_inside_top_left_area(tp, t))
-				tp_button_handle_event(tp, t, BUTTON_EVENT_IN_TOP_L, time);
+				event = BUTTON_EVENT_IN_TOP_L;
 			else
-				tp_button_handle_event(tp, t, BUTTON_EVENT_IN_AREA, time);
+				event = BUTTON_EVENT_IN_AREA;
+
+			tp_button_handle_event(tp, t, event, time);
 		}
 		if (tp->queued & TOUCHPAD_EVENT_BUTTON_RELEASE)
 			tp_button_handle_event(tp, t, BUTTON_EVENT_RELEASE, time);
@@ -461,7 +480,7 @@ tp_process_button(struct tp_dispatch *tp,
 		  const struct input_event *e,
 		  uint64_t time)
 {
-	struct libinput *libinput = tp->device->base.seat->libinput;
+	struct libinput *libinput = tp_libinput_context(tp);
 	uint32_t mask = 1 << (e->code - BTN_LEFT);
 
 	/* Ignore other buttons on clickpads */
@@ -508,8 +527,8 @@ tp_init_softbuttons(struct tp_dispatch *tp,
 	xoffset = absinfo_x->minimum,
 	yoffset = absinfo_y->minimum;
 	yres = absinfo_y->resolution;
-	width = abs(absinfo_x->maximum - absinfo_x->minimum);
-	height = abs(absinfo_y->maximum - absinfo_y->minimum);
+	width = device->abs.dimensions.x;
+	height = device->abs.dimensions.y;
 
 	/* button height: 10mm or 15% of the touchpad height,
 	   whichever is smaller */
@@ -539,8 +558,8 @@ tp_init_top_softbuttons(struct tp_dispatch *tp,
 	xoffset = absinfo_x->minimum,
 	yoffset = absinfo_y->minimum;
 	yres = absinfo_y->resolution;
-	width = abs(absinfo_x->maximum - absinfo_x->minimum);
-	height = abs(absinfo_y->maximum - absinfo_y->minimum);
+	width = device->abs.dimensions.x;
+	height = device->abs.dimensions.y;
 
 	if (tp->buttons.has_topbuttons) {
 		/* T440s has the top button line 5mm from the top, event
@@ -629,12 +648,25 @@ tp_button_config_click_get_method(struct libinput_device *device)
 static enum libinput_config_click_method
 tp_click_get_default_method(struct tp_dispatch *tp)
 {
+	struct evdev_device *device = tp->device;
+
 	if (!tp->buttons.is_clickpad)
 		return LIBINPUT_CONFIG_CLICK_METHOD_NONE;
 	else if (libevdev_get_id_vendor(tp->device->evdev) == VENDOR_ID_APPLE)
 		return LIBINPUT_CONFIG_CLICK_METHOD_CLICKFINGER;
-	else
-		return LIBINPUT_CONFIG_CLICK_METHOD_BUTTON_AREAS;
+
+	switch (device->model) {
+	case EVDEV_MODEL_CHROMEBOOK:
+	case EVDEV_MODEL_SYSTEM76_BONOBO:
+	case EVDEV_MODEL_SYSTEM76_GALAGO:
+	case EVDEV_MODEL_SYSTEM76_KUDU:
+	case EVDEV_MODEL_CLEVO_W740SU:
+		return LIBINPUT_CONFIG_CLICK_METHOD_CLICKFINGER;
+	default:
+		break;
+	}
+
+	return LIBINPUT_CONFIG_CLICK_METHOD_BUTTON_AREAS;
 }
 
 static enum libinput_config_click_method
@@ -646,11 +678,40 @@ tp_button_config_click_get_default_method(struct libinput_device *device)
 	return tp_click_get_default_method(tp);
 }
 
+static inline void
+tp_init_middlebutton_emulation(struct tp_dispatch *tp,
+			       struct evdev_device *device)
+{
+	bool enable_by_default,
+	     want_config_option;
+
+	if (tp->buttons.is_clickpad)
+		return;
+
+	/* init middle button emulation on non-clickpads, but only if we
+	 * don't have a middle button. Exception: ALPS touchpads don't know
+	 * if they have a middle button, so we always want the option there
+	 * and enabled by default.
+	 */
+	if (!libevdev_has_event_code(device->evdev, EV_KEY, BTN_MIDDLE)) {
+		enable_by_default = true;
+		want_config_option = false;
+	} else if (device->model == EVDEV_MODEL_ALPS_TOUCHPAD) {
+		enable_by_default = true;
+		want_config_option = true;
+	} else
+		return;
+
+	evdev_init_middlebutton(tp->device,
+				enable_by_default,
+				want_config_option);
+}
+
 int
 tp_init_buttons(struct tp_dispatch *tp,
 		struct evdev_device *device)
 {
-	struct libinput *libinput = tp->device->base.seat->libinput;
+	struct libinput *libinput = tp_libinput_context(tp);
 	struct tp_touch *t;
 	int width, height;
 	double diagonal;
@@ -677,11 +738,19 @@ tp_init_buttons(struct tp_dispatch *tp,
 	absinfo_x = device->abs.absinfo_x;
 	absinfo_y = device->abs.absinfo_y;
 
-	width = abs(absinfo_x->maximum - absinfo_x->minimum);
-	height = abs(absinfo_y->maximum - absinfo_y->minimum);
-	diagonal = sqrt(width*width + height*height);
-
-	tp->buttons.motion_dist = diagonal * DEFAULT_BUTTON_MOTION_THRESHOLD;
+	/* pinned-finger motion threshold, see tp_unpin_finger.
+	   The MAGIC for resolution-less touchpads ends up as 2% of the diagonal */
+	if (device->abs.fake_resolution) {
+		const double BUTTON_MOTION_MAGIC = 0.007;
+		width = device->abs.dimensions.x;
+		height = device->abs.dimensions.y;
+		diagonal = sqrt(width*width + height*height);
+		tp->buttons.motion_dist.x_scale_coeff = diagonal * BUTTON_MOTION_MAGIC;
+		tp->buttons.motion_dist.y_scale_coeff = diagonal * BUTTON_MOTION_MAGIC;
+	} else {
+		tp->buttons.motion_dist.x_scale_coeff = 1.0/absinfo_x->resolution;
+		tp->buttons.motion_dist.y_scale_coeff = 1.0/absinfo_y->resolution;
+	}
 
 	tp->buttons.config_method.get_methods = tp_button_config_click_get_methods;
 	tp->buttons.config_method.set_method = tp_button_config_click_set_method;
@@ -694,10 +763,12 @@ tp_init_buttons(struct tp_dispatch *tp,
 
 	tp_init_top_softbuttons(tp, device, 1.0);
 
+	tp_init_middlebutton_emulation(tp, device);
+
 	tp_for_each_touch(tp, t) {
 		t->button.state = BUTTON_STATE_NONE;
 		libinput_timer_init(&t->button.timer,
-				    tp->device->base.seat->libinput,
+				    tp_libinput_context(tp),
 				    tp_button_handle_timeout, t);
 	}
 
@@ -734,10 +805,10 @@ tp_post_physical_buttons(struct tp_dispatch *tp, uint64_t time)
 				state = LIBINPUT_BUTTON_STATE_RELEASED;
 
 			b = evdev_to_left_handed(tp->device, button);
-			evdev_pointer_notify_button(tp->device,
-						    time,
-						    b,
-						    state);
+			evdev_pointer_notify_physical_button(tp->device,
+							     time,
+							     b,
+							     state);
 		}
 
 		button++;
@@ -748,6 +819,102 @@ tp_post_physical_buttons(struct tp_dispatch *tp, uint64_t time)
 	return 0;
 }
 
+static inline int
+tp_check_clickfinger_distance(struct tp_dispatch *tp,
+			      struct tp_touch *t1,
+			      struct tp_touch *t2)
+{
+	double x, y;
+
+	if (!t1 || !t2)
+		return 0;
+
+	x = abs(t1->point.x - t2->point.x);
+	y = abs(t1->point.y - t2->point.y);
+
+	/* no resolution, so let's assume they're close enough together */
+	if (tp->device->abs.fake_resolution) {
+		int w, h;
+
+		/* Use a maximum of 30% of the touchpad width or height if
+		 * we dont' have resolution. */
+		w = tp->device->abs.dimensions.x;
+		h = tp->device->abs.dimensions.y;
+
+		return (x < w * 0.3 && y < h * 0.3) ? 1 : 0;
+	} else {
+		/* maximum spread is 40mm horiz, 20mm vert. Anything wider than that
+		 * is probably a gesture. The y spread is small so we ignore clicks
+		 * with thumbs at the bottom of the touchpad while the pointer
+		 * moving finger is still on the pad */
+
+		x /= tp->device->abs.absinfo_x->resolution;
+		y /= tp->device->abs.absinfo_y->resolution;
+
+		return (x < 40 && y < 20) ? 1 : 0;
+	}
+
+}
+
+static uint32_t
+tp_clickfinger_set_button(struct tp_dispatch *tp)
+{
+	uint32_t button;
+	unsigned int nfingers = tp->nfingers_down;
+	struct tp_touch *t;
+	struct tp_touch *first = NULL,
+			*second = NULL,
+			*third = NULL;
+	uint32_t close_touches = 0;
+
+	if (nfingers < 2 || nfingers > 3)
+		goto out;
+
+	/* two or three fingers down on the touchpad. Check for distance
+	 * between the fingers. */
+	tp_for_each_touch(tp, t) {
+		if (t->state != TOUCH_BEGIN && t->state != TOUCH_UPDATE)
+			continue;
+
+		if (!first)
+			first = t;
+		else if (!second)
+			second = t;
+		else if (!third) {
+			third = t;
+			break;
+		}
+	}
+
+	if (!first || !second) {
+		nfingers = 1;
+		goto out;
+	}
+
+	close_touches |= tp_check_clickfinger_distance(tp, first, second) << 0;
+	close_touches |= tp_check_clickfinger_distance(tp, second, third) << 1;
+	close_touches |= tp_check_clickfinger_distance(tp, first, third) << 2;
+
+	switch(__builtin_popcount(close_touches)) {
+	case 0: nfingers = 1; break;
+	case 1: nfingers = 2; break;
+	default: nfingers = 3; break;
+	}
+
+out:
+	switch (nfingers) {
+	case 0:
+	case 1: button = BTN_LEFT; break;
+	case 2: button = BTN_RIGHT; break;
+	case 3: button = BTN_MIDDLE; break;
+	default:
+		button = 0;
+		break;
+	}
+
+	return button;
+}
+
 static int
 tp_notify_clickpadbutton(struct tp_dispatch *tp,
 			 uint64_t time,
@@ -765,8 +932,10 @@ tp_notify_clickpadbutton(struct tp_dispatch *tp,
 		event.type = EV_KEY;
 		event.code = button;
 		event.value = (state == LIBINPUT_BUTTON_STATE_PRESSED) ? 1 : 0;
-		dispatch->interface->process(dispatch, tp->buttons.trackpoint,
-					     &event, time);
+		dispatch->interface->process(dispatch,
+					     tp->buttons.trackpoint,
+					     &event,
+					     time);
 		return 1;
 	}
 
@@ -779,14 +948,8 @@ tp_notify_clickpadbutton(struct tp_dispatch *tp,
 	 * by the softbutton code with one based on the number of fingers.
 	 */
 	if (tp->buttons.click_method == LIBINPUT_CONFIG_CLICK_METHOD_CLICKFINGER &&
-			state == LIBINPUT_BUTTON_STATE_PRESSED) {
-		switch (tp->nfingers_down) {
-		case 1: button = BTN_LEFT; break;
-		case 2: button = BTN_RIGHT; break;
-		case 3: button = BTN_MIDDLE; break;
-		default:
-			button = 0;
-		}
+	    state == LIBINPUT_BUTTON_STATE_PRESSED) {
+		button = tp_clickfinger_set_button(tp);
 		tp->buttons.active = button;
 
 		if (!button)
@@ -814,46 +977,48 @@ tp_post_clickpadbutton_buttons(struct tp_dispatch *tp, uint64_t time)
 
 	if (current) {
 		struct tp_touch *t;
+		uint32_t area = 0;
 
 		tp_for_each_touch(tp, t) {
 			switch (t->button.curr) {
 			case BUTTON_EVENT_IN_AREA:
-				button |= AREA;
+				area |= AREA;
 				break;
 			case BUTTON_EVENT_IN_TOP_L:
 				is_top = 1;
 				/* fallthrough */
 			case BUTTON_EVENT_IN_BOTTOM_L:
-				button |= LEFT;
+				area |= LEFT;
 				break;
 			case BUTTON_EVENT_IN_TOP_M:
 				is_top = 1;
-				button |= MIDDLE;
+				area |= MIDDLE;
 				break;
 			case BUTTON_EVENT_IN_TOP_R:
 				is_top = 1;
 				/* fallthrough */
 			case BUTTON_EVENT_IN_BOTTOM_R:
-				button |= RIGHT;
+				area |= RIGHT;
 				break;
 			default:
 				break;
 			}
 		}
 
-		if (button == 0) {
+		if (area == 0 &&
+		    tp->buttons.click_method != LIBINPUT_CONFIG_CLICK_METHOD_CLICKFINGER) {
 			/* No touches, wait for a touch before processing */
 			tp->buttons.click_pending = true;
 			return 0;
 		}
 
-		if ((button & MIDDLE) || ((button & LEFT) && (button & RIGHT)))
+		if ((area & MIDDLE) || ((area & LEFT) && (area & RIGHT)))
 			button = evdev_to_left_handed(tp->device, BTN_MIDDLE);
-		else if (button & RIGHT)
+		else if (area & RIGHT)
 			button = evdev_to_left_handed(tp->device, BTN_RIGHT);
-		else if (button & LEFT)
+		else if (area & LEFT)
 			button = evdev_to_left_handed(tp->device, BTN_LEFT);
-		else /* main area is always BTN_LEFT */
+		else /* main or no area (for clickfinger) is always BTN_LEFT */
 			button = BTN_LEFT;
 
 		tp->buttons.active = button;
@@ -870,8 +1035,11 @@ tp_post_clickpadbutton_buttons(struct tp_dispatch *tp, uint64_t time)
 	tp->buttons.click_pending = false;
 
 	if (button)
-		return tp_notify_clickpadbutton(tp, time, button, is_top, state);
-
+		return tp_notify_clickpadbutton(tp,
+						time,
+						button,
+						is_top,
+						state);
 	return 0;
 }
 
@@ -893,5 +1061,6 @@ tp_button_touch_active(struct tp_dispatch *tp, struct tp_touch *t)
 bool
 tp_button_is_inside_softbutton_area(struct tp_dispatch *tp, struct tp_touch *t)
 {
-	return is_inside_top_button_area(tp, t) || is_inside_bottom_button_area(tp, t);
+	return is_inside_top_button_area(tp, t) ||
+	       is_inside_bottom_button_area(tp, t);
 }
