@@ -1,25 +1,28 @@
 /*
- * Copyright © 2014 Red Hat, Inc.
+ * Copyright © 2014-2015 Red Hat, Inc.
  *
- * Permission to use, copy, modify, distribute, and sell this software and
- * its documentation for any purpose is hereby granted without fee, provided
- * that the above copyright notice appear in all copies and that both that
- * copyright notice and this permission notice appear in supporting
- * documentation, and that the name of the copyright holders not be used in
- * advertising or publicity pertaining to distribution of the software
- * without specific, written prior permission.  The copyright holders make
- * no representations about the suitability of this software for any
- * purpose.  It is provided "as is" without express or implied warranty.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
- * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
- * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
- * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
- * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
- * CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
- * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ * The above copyright notice and this permission notice (including the next
+ * paragraph) shall be included in all copies or substantial portions of the
+ * Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
+#include "config.h"
+
 #include <errno.h>
 #include <limits.h>
 #include <math.h>
@@ -29,12 +32,14 @@
 
 #include "evdev-mt-touchpad.h"
 
+#define CASE_RETURN_STRING(a) case a: return #a
+
 #define DEFAULT_SCROLL_LOCK_TIMEOUT 300 /* ms */
 /* Use a reasonably large threshold until locked into scrolling mode, to
    avoid accidentally locking in scrolling mode when trying to use the entire
    touchpad to move the pointer. The user can wait for the timeout to trigger
    to do a small scroll. */
-#define DEFAULT_SCROLL_THRESHOLD TP_MM_TO_DPI_NORMALIZED(5)
+#define DEFAULT_SCROLL_THRESHOLD TP_MM_TO_DPI_NORMALIZED(3)
 
 enum scroll_event {
 	SCROLL_EVENT_TOUCH,
@@ -44,7 +49,33 @@ enum scroll_event {
 	SCROLL_EVENT_POSTED,
 };
 
-static uint32_t
+static inline const char*
+edge_state_to_str(enum tp_edge_scroll_touch_state state)
+{
+
+	switch (state) {
+	CASE_RETURN_STRING(EDGE_SCROLL_TOUCH_STATE_NONE);
+	CASE_RETURN_STRING(EDGE_SCROLL_TOUCH_STATE_EDGE_NEW);
+	CASE_RETURN_STRING(EDGE_SCROLL_TOUCH_STATE_EDGE);
+	CASE_RETURN_STRING(EDGE_SCROLL_TOUCH_STATE_AREA);
+	}
+	return NULL;
+}
+
+static inline const char*
+edge_event_to_str(enum scroll_event event)
+{
+	switch (event) {
+	CASE_RETURN_STRING(SCROLL_EVENT_TOUCH);
+	CASE_RETURN_STRING(SCROLL_EVENT_MOTION);
+	CASE_RETURN_STRING(SCROLL_EVENT_RELEASE);
+	CASE_RETURN_STRING(SCROLL_EVENT_TIMEOUT);
+	CASE_RETURN_STRING(SCROLL_EVENT_POSTED);
+	}
+	return NULL;
+}
+
+uint32_t
 tp_touch_get_edge(struct tp_dispatch *tp, struct tp_touch *t)
 {
 	uint32_t edge = EDGE_NONE;
@@ -93,7 +124,7 @@ tp_edge_scroll_handle_none(struct tp_dispatch *tp,
 			   struct tp_touch *t,
 			   enum scroll_event event)
 {
-	struct libinput *libinput = tp->device->base.seat->libinput;
+	struct libinput *libinput = tp_libinput_context(tp);
 
 	switch (event) {
 	case SCROLL_EVENT_TOUCH:
@@ -121,7 +152,7 @@ tp_edge_scroll_handle_edge_new(struct tp_dispatch *tp,
 			       struct tp_touch *t,
 			       enum scroll_event event)
 {
-	struct libinput *libinput = tp->device->base.seat->libinput;
+	struct libinput *libinput = tp_libinput_context(tp);
 
 	switch (event) {
 	case SCROLL_EVENT_TOUCH:
@@ -150,7 +181,7 @@ tp_edge_scroll_handle_edge(struct tp_dispatch *tp,
 			   struct tp_touch *t,
 			   enum scroll_event event)
 {
-	struct libinput *libinput = tp->device->base.seat->libinput;
+	struct libinput *libinput = tp_libinput_context(tp);
 
 	switch (event) {
 	case SCROLL_EVENT_TOUCH:
@@ -181,7 +212,7 @@ tp_edge_scroll_handle_area(struct tp_dispatch *tp,
 			   struct tp_touch *t,
 			   enum scroll_event event)
 {
-	struct libinput *libinput = tp->device->base.seat->libinput;
+	struct libinput *libinput = tp_libinput_context(tp);
 
 	switch (event) {
 	case SCROLL_EVENT_TOUCH:
@@ -204,7 +235,10 @@ tp_edge_scroll_handle_event(struct tp_dispatch *tp,
 			    struct tp_touch *t,
 			    enum scroll_event event)
 {
-	switch (t->scroll.edge_state) {
+	struct libinput *libinput = tp_libinput_context(tp);
+	enum tp_edge_scroll_touch_state current = t->scroll.edge_state;
+
+	switch (current) {
 	case EDGE_SCROLL_TOUCH_STATE_NONE:
 		tp_edge_scroll_handle_none(tp, t, event);
 		break;
@@ -218,6 +252,12 @@ tp_edge_scroll_handle_event(struct tp_dispatch *tp,
 		tp_edge_scroll_handle_area(tp, t, event);
 		break;
 	}
+
+	log_debug(libinput,
+		  "edge state: %s → %s → %s\n",
+		  edge_state_to_str(current),
+		  edge_event_to_str(event),
+		  edge_state_to_str(t->scroll.edge_state));
 }
 
 static void
@@ -235,8 +275,8 @@ tp_edge_scroll_init(struct tp_dispatch *tp, struct evdev_device *device)
 	int width, height;
 	int edge_width, edge_height;
 
-	width = device->abs.absinfo_x->maximum - device->abs.absinfo_x->minimum;
-	height = device->abs.absinfo_y->maximum - device->abs.absinfo_y->minimum;
+	width = device->abs.dimensions.x;
+	height = device->abs.dimensions.y;
 
 	switch (tp->model) {
 	case MODEL_ALPS:
@@ -264,7 +304,7 @@ tp_edge_scroll_init(struct tp_dispatch *tp, struct evdev_device *device)
 	tp_for_each_touch(tp, t) {
 		t->scroll.direction = -1;
 		libinput_timer_init(&t->scroll.timer,
-				    device->base.seat->libinput,
+				    tp_libinput_context(tp),
 				    tp_edge_scroll_handle_timeout, t);
 	}
 
@@ -313,8 +353,7 @@ tp_edge_scroll_post_events(struct tp_dispatch *tp, uint64_t time)
 	struct tp_touch *t;
 	enum libinput_pointer_axis axis;
 	double *delta;
-	double initial_dx, initial_dy, *initial_delta;
-	struct normalized_coords normalized;
+	struct normalized_coords normalized, tmp;
 	const struct normalized_coords zero = { 0.0, 0.0 };
 	const struct discrete_coords zero_discrete = { 0.0, 0.0 };
 
@@ -325,6 +364,9 @@ tp_edge_scroll_post_events(struct tp_dispatch *tp, uint64_t time)
 		if (!t->dirty)
 			continue;
 
+		if (t->palm.state != PALM_NONE)
+			continue;
+
 		switch (t->scroll.edge) {
 			case EDGE_NONE:
 				if (t->scroll.direction != -1) {
@@ -340,20 +382,17 @@ tp_edge_scroll_post_events(struct tp_dispatch *tp, uint64_t time)
 			case EDGE_RIGHT:
 				axis = LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL;
 				delta = &normalized.y;
-				initial_delta = &initial_dy;
 				break;
 			case EDGE_BOTTOM:
 				axis = LIBINPUT_POINTER_AXIS_SCROLL_HORIZONTAL;
 				delta = &normalized.x;
-				initial_delta = &initial_dx;
 				break;
 			default: /* EDGE_RIGHT | EDGE_BOTTOM */
 				continue; /* Don't know direction yet, skip */
 		}
 
 		normalized = tp_get_delta(t);
-		tp_filter_motion(tp, &normalized.x, &normalized.y,
-				 NULL, NULL, time);
+		normalized = tp_filter_motion(tp, &normalized, time);
 
 		switch (t->scroll.edge_state) {
 		case EDGE_SCROLL_TOUCH_STATE_NONE:
@@ -363,16 +402,14 @@ tp_edge_scroll_post_events(struct tp_dispatch *tp, uint64_t time)
 					 t->scroll.edge_state);
 			break;
 		case EDGE_SCROLL_TOUCH_STATE_EDGE_NEW:
-			initial_dx = t->point.x - t->scroll.initial.x;
-			initial_dy = t->point.y - t->scroll.initial.y;
-			tp_normalize_delta(tp,
-					   initial_dx,
-					   initial_dy,
-					   &normalized);
-			if (fabs(*initial_delta) < DEFAULT_SCROLL_THRESHOLD) {
-				normalized.x = 0.0;
-				normalized.y = 0.0;
-			}
+			tmp = normalized;
+			normalized = tp_normalize_delta(tp,
+					device_delta(t->point,
+						     t->scroll.initial));
+			if (fabs(*delta) < DEFAULT_SCROLL_THRESHOLD)
+				normalized = zero;
+			else
+				normalized = tmp;
 			break;
 		case EDGE_SCROLL_TOUCH_STATE_EDGE:
 			break;
@@ -410,6 +447,10 @@ tp_edge_scroll_stop_events(struct tp_dispatch *tp, uint64_t time)
 					    &zero,
 					    &zero_discrete);
 			t->scroll.direction = -1;
+			/* reset touch to area state, avoids loading the
+			 * state machine with special case handling */
+			t->scroll.edge = EDGE_NONE;
+			t->scroll.edge_state = EDGE_SCROLL_TOUCH_STATE_AREA;
 		}
 	}
 }
