@@ -1,23 +1,24 @@
 /*
  * Copyright © 2015 Red Hat, Inc.
  *
- * Permission to use, copy, modify, distribute, and sell this software and
- * its documentation for any purpose is hereby granted without fee, provided
- * that the above copyright notice appear in all copies and that both that
- * copyright notice and this permission notice appear in supporting
- * documentation, and that the name of the copyright holders not be used in
- * advertising or publicity pertaining to distribution of the software
- * without specific, written prior permission.  The copyright holders make
- * no representations about the suitability of this software for any
- * purpose.  It is provided "as is" without express or implied warranty.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
- * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
- * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
- * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
- * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
- * CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
- * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ * The above copyright notice and this permission notice (including the next
+ * paragraph) shall be included in all copies or substantial portions of the
+ * Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #include "config.h"
@@ -39,7 +40,7 @@ tp_get_touches_delta(struct tp_dispatch *tp, bool average)
 	struct normalized_coords normalized;
 	struct normalized_coords delta = {0.0, 0.0};
 
-	for (i = 0; i < tp->real_touches; i++) {
+	for (i = 0; i < tp->num_slots; i++) {
 		t = &tp->touches[i];
 
 		if (tp_touch_active(tp, t) && t->dirty) {
@@ -90,19 +91,22 @@ static void
 tp_gesture_post_pointer_motion(struct tp_dispatch *tp, uint64_t time)
 {
 	struct normalized_coords delta, unaccel;
+	struct device_float_coords raw;
 
 	/* When a clickpad is clicked, combine motion of all active touches */
 	if (tp->buttons.is_clickpad && tp->buttons.state)
-		delta = tp_get_combined_touches_delta(tp);
+		unaccel = tp_get_combined_touches_delta(tp);
 	else
-		delta = tp_get_average_touches_delta(tp);
+		unaccel = tp_get_average_touches_delta(tp);
 
-	tp_filter_motion(tp, &delta.x, &delta.y, &unaccel.x, &unaccel.y, time);
+	delta = tp_filter_motion(tp, &unaccel, time);
 
-	if (delta.x != 0.0 || delta.y != 0.0 ||
-	    unaccel.x != 0.0 || unaccel.y != 0.0) {
-		pointer_notify_motion(&tp->device->base, time,
-				      &delta, &unaccel);
+	if (!normalized_is_zero(delta) || !normalized_is_zero(unaccel)) {
+		raw = tp_unnormalize_for_xaxis(tp, unaccel);
+		pointer_notify_motion(&tp->device->base,
+				      time,
+				      &delta,
+				      &raw);
 	}
 }
 
@@ -111,10 +115,23 @@ tp_gesture_post_twofinger_scroll(struct tp_dispatch *tp, uint64_t time)
 {
 	struct normalized_coords delta;
 
-	delta = tp_get_average_touches_delta(tp);
-	tp_filter_motion(tp, &delta.x, &delta.y, NULL, NULL, time);
+	if (tp->scroll.method != LIBINPUT_CONFIG_SCROLL_2FG)
+		return;
 
-	if (delta.x == 0.0 && delta.y == 0.0)
+	/* On some semi-mt models slot 0 is more accurate, so for semi-mt
+	 * we only use slot 0. */
+	if (tp->semi_mt) {
+		if (!tp->touches[0].dirty)
+			return;
+
+		delta = tp_get_delta(&tp->touches[0]);
+	} else {
+		delta = tp_get_average_touches_delta(tp);
+	}
+
+	delta = tp_filter_motion(tp, &delta, time);
+
+	if (normalized_is_zero(delta))
 		return;
 
 	tp_gesture_start(tp, time);
@@ -154,6 +171,9 @@ tp_gesture_post_events(struct tp_dispatch *tp, uint64_t time)
 void
 tp_gesture_stop_twofinger_scroll(struct tp_dispatch *tp, uint64_t time)
 {
+	if (tp->scroll.method != LIBINPUT_CONFIG_SCROLL_2FG)
+		return;
+
 	evdev_stop_scroll(tp->device,
 			  time,
 			  LIBINPUT_POINTER_AXIS_SOURCE_FINGER);
