@@ -1,23 +1,24 @@
 /*
  * Copyright © 2014 Red Hat, Inc.
  *
- * Permission to use, copy, modify, distribute, and sell this software and
- * its documentation for any purpose is hereby granted without fee, provided
- * that the above copyright notice appear in all copies and that both that
- * copyright notice and this permission notice appear in supporting
- * documentation, and that the name of the copyright holders not be used in
- * advertising or publicity pertaining to distribution of the software
- * without specific, written prior permission.  The copyright holders make
- * no representations about the suitability of this software for any
- * purpose.  It is provided "as is" without express or implied warranty.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
- * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
- * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
- * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
- * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
- * CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
- * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ * The above copyright notice and this permission notice (including the next
+ * paragraph) shall be included in all copies or substantial portions of the
+ * Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #include <config.h>
@@ -69,7 +70,7 @@ START_TEST(device_sendevents_config_touchpad)
 	expected = LIBINPUT_CONFIG_SEND_EVENTS_DISABLED;
 
 	/* The wacom devices in the test suite are external */
-	if (libevdev_get_id_vendor(dev->evdev) != 0x56a) /* wacom */
+	if (libevdev_get_id_vendor(dev->evdev) != VENDOR_ID_WACOM)
 		expected |=
 			LIBINPUT_CONFIG_SEND_EVENTS_DISABLED_ON_EXTERNAL_MOUSE;
 
@@ -313,6 +314,9 @@ START_TEST(device_reenable_syspath_changed)
 
 	litest_device = litest_add_device(li, LITEST_MOUSE);
 	device2 = litest_device->libinput_device;
+	/* Note: if the sysname isn't the same, some other device got added
+	 * or removed while this test was running.  This is unlikely and
+	 * would result in a false positive, so let's fail the test here */
 	ck_assert_str_eq(libinput_device_get_sysname(device1),
 			 libinput_device_get_sysname(device2));
 
@@ -814,37 +818,39 @@ START_TEST(abs_mt_device_no_absx)
 }
 END_TEST
 
-START_TEST(abs_device_no_range)
+static void
+assert_device_ignored(struct libinput *li, struct input_absinfo *absinfo)
 {
 	struct libevdev_uinput *uinput;
-	struct libinput *li;
 	struct libinput_device *device;
-	int code;
+
+	uinput = litest_create_uinput_abs_device("test device", NULL,
+						 absinfo,
+						 EV_KEY, BTN_LEFT,
+						 EV_KEY, BTN_RIGHT,
+						 -1);
+	device = libinput_path_add_device(li,
+					  libevdev_uinput_get_devnode(uinput));
+	litest_assert_ptr_null(device);
+	libevdev_uinput_destroy(uinput);
+}
+
+START_TEST(abs_device_no_range)
+{
+	struct libinput *li;
+	int code = _i; /* looped test */
 	/* set x/y so libinput doesn't just reject for missing axes */
 	struct input_absinfo absinfo[] = {
 		{ ABS_X, 0, 10, 0, 0, 0 },
 		{ ABS_Y, 0, 10, 0, 0, 0 },
-		{ -1, 0, 0, 0, 0, 0 },
+		{ code, 0, 0, 0, 0, 0 },
 		{ -1, -1, -1, -1, -1, -1 }
 	};
 
 	li = litest_create_context();
 	litest_disable_log_handler(li);
 
-	for (code = 0; code < ABS_MT_SLOT; code++) {
-		if (code == ABS_MISC)
-			continue;
-		absinfo[2].value = code;
-		uinput = litest_create_uinput_abs_device("test device", NULL,
-							 absinfo,
-							 EV_KEY, BTN_LEFT,
-							 EV_KEY, BTN_RIGHT,
-							 -1);
-		device = libinput_path_add_device(li,
-						  libevdev_uinput_get_devnode(uinput));
-		ck_assert(device == NULL);
-		libevdev_uinput_destroy(uinput);
-	}
+	assert_device_ignored(li, absinfo);
 
 	litest_restore_log_handler(li);
 	libinput_unref(li);
@@ -853,10 +859,8 @@ END_TEST
 
 START_TEST(abs_mt_device_no_range)
 {
-	struct libevdev_uinput *uinput;
 	struct libinput *li;
-	struct libinput_device *device;
-	int code;
+	int code = _i; /* looped test */
 	/* set x/y so libinput doesn't just reject for missing axes */
 	struct input_absinfo absinfo[] = {
 		{ ABS_X, 0, 10, 0, 0, 0 },
@@ -865,37 +869,149 @@ START_TEST(abs_mt_device_no_range)
 		{ ABS_MT_TRACKING_ID, 0, 255, 0, 0, 0 },
 		{ ABS_MT_POSITION_X, 0, 10, 0, 0, 0 },
 		{ ABS_MT_POSITION_Y, 0, 10, 0, 0, 0 },
-		{ -1, 0, 0, 0, 0, 0 },
+		{ code, 0, 0, 0, 0, 0 },
 		{ -1, -1, -1, -1, -1, -1 }
 	};
 
 	li = litest_create_context();
 	litest_disable_log_handler(li);
 
-	for (code = ABS_MT_SLOT + 1; code < ABS_CNT; code++) {
-		if (code == ABS_MT_TOOL_TYPE ||
-		    code == ABS_MT_TRACKING_ID) /* kernel overrides it */
-			continue;
-
-		absinfo[6].value = code;
-		uinput = litest_create_uinput_abs_device("test device", NULL,
-							 absinfo,
-							 EV_KEY, BTN_LEFT,
-							 EV_KEY, BTN_RIGHT,
-							 -1);
-		device = libinput_path_add_device(li,
-						  libevdev_uinput_get_devnode(uinput));
-		ck_assert(device == NULL);
-		libevdev_uinput_destroy(uinput);
-	}
+	if (code != ABS_MT_TOOL_TYPE &&
+	    code != ABS_MT_TRACKING_ID) /* kernel overrides it */
+		assert_device_ignored(li, absinfo);
 
 	litest_restore_log_handler(li);
 	libinput_unref(li);
 }
 END_TEST
 
-int main (int argc, char **argv)
+START_TEST(abs_device_missing_res)
 {
+	struct libinput *li;
+	struct input_absinfo absinfo[] = {
+		{ ABS_X, 0, 10, 0, 0, 10 },
+		{ ABS_Y, 0, 10, 0, 0, 0 },
+		{ -1, -1, -1, -1, -1, -1 }
+	};
+
+	li = litest_create_context();
+	litest_disable_log_handler(li);
+
+	assert_device_ignored(li, absinfo);
+
+	absinfo[0].resolution = 0;
+	absinfo[1].resolution = 20;
+
+	assert_device_ignored(li, absinfo);
+
+	litest_restore_log_handler(li);
+	libinput_unref(li);
+}
+END_TEST
+
+START_TEST(abs_mt_device_missing_res)
+{
+	struct libinput *li;
+	struct input_absinfo absinfo[] = {
+		{ ABS_X, 0, 10, 0, 0, 10 },
+		{ ABS_Y, 0, 10, 0, 0, 10 },
+		{ ABS_MT_SLOT, 0, 2, 0, 0, 0 },
+		{ ABS_MT_TRACKING_ID, 0, 255, 0, 0, 0 },
+		{ ABS_MT_POSITION_X, 0, 10, 0, 0, 10 },
+		{ ABS_MT_POSITION_Y, 0, 10, 0, 0, 0 },
+		{ -1, -1, -1, -1, -1, -1 }
+	};
+
+	li = litest_create_context();
+	litest_disable_log_handler(li);
+	assert_device_ignored(li, absinfo);
+
+	absinfo[4].resolution = 0;
+	absinfo[5].resolution = 20;
+
+	assert_device_ignored(li, absinfo);
+
+	litest_restore_log_handler(li);
+	libinput_unref(li);
+
+}
+END_TEST
+
+START_TEST(device_wheel_only)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput_device *device = dev->libinput_device;
+
+	ck_assert(libinput_device_has_capability(device,
+						 LIBINPUT_DEVICE_CAP_POINTER));
+}
+END_TEST
+
+START_TEST(device_udev_tag_alps)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput_device *device = dev->libinput_device;
+	struct udev_device *d;
+	const char *prop;
+
+	d = libinput_device_get_udev_device(device);
+	prop = udev_device_get_property_value(d,
+					      "LIBINPUT_MODEL_ALPS_TOUCHPAD");
+
+	if (strstr(libinput_device_get_name(device), "ALPS"))
+		ck_assert_notnull(prop);
+	else
+		ck_assert(prop == NULL);
+
+	udev_device_unref(d);
+}
+END_TEST
+
+START_TEST(device_udev_tag_wacom)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput_device *device = dev->libinput_device;
+	struct udev_device *d;
+	const char *prop;
+
+	d = libinput_device_get_udev_device(device);
+	prop = udev_device_get_property_value(d,
+					      "LIBINPUT_MODEL_WACOM_TOUCHPAD");
+
+	if (libevdev_get_id_vendor(dev->evdev) == VENDOR_ID_WACOM)
+		ck_assert_notnull(prop);
+	else
+		ck_assert(prop == NULL);
+
+	udev_device_unref(d);
+}
+END_TEST
+
+START_TEST(device_udev_tag_apple)
+{
+	struct litest_device *dev = litest_current_device();
+	struct libinput_device *device = dev->libinput_device;
+	struct udev_device *d;
+	const char *prop;
+
+	d = libinput_device_get_udev_device(device);
+	prop = udev_device_get_property_value(d,
+					      "LIBINPUT_MODEL_WACOM_TOUCHPAD");
+
+	if (libevdev_get_id_vendor(dev->evdev) == VENDOR_ID_WACOM)
+		ck_assert_notnull(prop);
+	else
+		ck_assert(prop == NULL);
+
+	udev_device_unref(d);
+}
+END_TEST
+void
+litest_setup_tests(void)
+{
+	struct range abs_range = { 0, ABS_MISC };
+	struct range abs_mt_range = { ABS_MT_SLOT + 1, ABS_CNT };
+
 	litest_add("device:sendevents", device_sendevents_config, LITEST_ANY, LITEST_TOUCHPAD);
 	litest_add("device:sendevents", device_sendevents_config_invalid, LITEST_ANY, LITEST_ANY);
 	litest_add("device:sendevents", device_sendevents_config_touchpad, LITEST_TOUCHPAD, LITEST_ANY);
@@ -926,8 +1042,14 @@ int main (int argc, char **argv)
 	litest_add_no_device("device:invalid devices", abs_device_no_absy);
 	litest_add_no_device("device:invalid devices", abs_mt_device_no_absx);
 	litest_add_no_device("device:invalid devices", abs_mt_device_no_absy);
-	litest_add_no_device("device:invalid devices", abs_device_no_range);
-	litest_add_no_device("device:invalid devices", abs_mt_device_no_range);
+	litest_add_ranged_no_device("device:invalid devices", abs_device_no_range, &abs_range);
+	litest_add_ranged_no_device("device:invalid devices", abs_mt_device_no_range, &abs_mt_range);
+	litest_add_no_device("device:invalid devices", abs_device_missing_res);
+	litest_add_no_device("device:invalid devices", abs_mt_device_missing_res);
 
-	return litest_run(argc, argv);
+	litest_add("device:wheel", device_wheel_only, LITEST_WHEEL, LITEST_RELATIVE|LITEST_ABSOLUTE);
+
+	litest_add("device:udev tags", device_udev_tag_alps, LITEST_TOUCHPAD, LITEST_ANY);
+	litest_add("device:udev tags", device_udev_tag_wacom, LITEST_TOUCHPAD, LITEST_ANY);
+	litest_add("device:udev tags", device_udev_tag_apple, LITEST_TOUCHPAD, LITEST_ANY);
 }
