@@ -1,23 +1,24 @@
 /*
  * Copyright © 2014 Red Hat, Inc.
  *
- * Permission to use, copy, modify, distribute, and sell this software and
- * its documentation for any purpose is hereby granted without fee, provided
- * that the above copyright notice appear in all copies and that both that
- * copyright notice and this permission notice appear in supporting
- * documentation, and that the name of the copyright holders not be used in
- * advertising or publicity pertaining to distribution of the software
- * without specific, written prior permission.  The copyright holders make
- * no representations about the suitability of this software for any
- * purpose.  It is provided "as is" without express or implied warranty.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
- * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
- * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
- * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
- * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
- * CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
- * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ * The above copyright notice and this permission notice (including the next
+ * paragraph) shall be included in all copies or substantial portions of the
+ * Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 
 #include <config.h>
@@ -64,7 +65,7 @@ create_simple_test_device(const char *name, ...)
 	};
 
 	evdev = libevdev_new();
-	ck_assert(evdev != NULL);
+	litest_assert_notnull(evdev);
 	libevdev_set_name(evdev, name);
 
 	va_start(args, name);
@@ -82,7 +83,7 @@ create_simple_test_device(const char *name, ...)
 	rc = libevdev_uinput_create_from_device(evdev,
 						LIBEVDEV_UINPUT_OPEN_MANAGED,
 						&uinput);
-	ck_assert_int_eq(rc, 0);
+	litest_assert_int_eq(rc, 0);
 	libevdev_free(evdev);
 
 	return uinput;
@@ -557,7 +558,35 @@ START_TEST(wheel_click_parser)
 }
 END_TEST
 
-int main (int argc, char **argv) {
+struct parser_test_float {
+	char *tag;
+	double expected_value;
+};
+
+START_TEST(trackpoint_accel_parser)
+{
+	struct parser_test_float tests[] = {
+		{ "0.5", 0.5 },
+		{ "1.0", 1.0 },
+		{ "2.0", 2.0 },
+		{ "fail1.0", 0.0 },
+		{ "1.0fail", 0.0 },
+		{ "0,5", 0.0 },
+		{ NULL, 0.0 }
+	};
+	int i;
+	double accel;
+
+	for (i = 0; tests[i].tag != NULL; i++) {
+		accel = parse_trackpoint_accel_property(tests[i].tag);
+		ck_assert(accel == tests[i].expected_value);
+	}
+}
+END_TEST
+
+void
+litest_setup_tests(void)
+{
 	litest_add_no_device("events:conversion", event_conversion_device_notify);
 	litest_add_for_device("events:conversion", event_conversion_pointer, LITEST_MOUSE);
 	litest_add_for_device("events:conversion", event_conversion_pointer, LITEST_MOUSE);
@@ -570,8 +599,7 @@ int main (int argc, char **argv) {
 
 	litest_add_no_device("misc:matrix", matrix_helpers);
 	litest_add_no_device("misc:ratelimit", ratelimit_helpers);
-	litest_add_no_device("misc:dpi parser", dpi_parser);
-	litest_add_no_device("misc:wheel click parser", wheel_click_parser);
-
-	return litest_run(argc, argv);
+	litest_add_no_device("misc:parser", dpi_parser);
+	litest_add_no_device("misc:parser", wheel_click_parser);
+	litest_add_no_device("misc:parser", trackpoint_accel_parser);
 }
