@@ -1,23 +1,24 @@
 /*
  * Copyright © 2014 Red Hat, Inc.
  *
- * Permission to use, copy, modify, distribute, and sell this software and
- * its documentation for any purpose is hereby granted without fee, provided
- * that the above copyright notice appear in all copies and that both that
- * copyright notice and this permission notice appear in supporting
- * documentation, and that the name of the copyright holders not be used in
- * advertising or publicity pertaining to distribution of the software
- * without specific, written prior permission.  The copyright holders make
- * no representations about the suitability of this software for any
- * purpose.  It is provided "as is" without express or implied warranty.
+ * Permission is hereby granted, free of charge, to any person obtaining a
+ * copy of this software and associated documentation files (the "Software"),
+ * to deal in the Software without restriction, including without limitation
+ * the rights to use, copy, modify, merge, publish, distribute, sublicense,
+ * and/or sell copies of the Software, and to permit persons to whom the
+ * Software is furnished to do so, subject to the following conditions:
  *
- * THE COPYRIGHT HOLDERS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS
- * SOFTWARE, INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND
- * FITNESS, IN NO EVENT SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY
- * SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER
- * RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF
- * CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
- * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
+ * The above copyright notice and this permission notice (including the next
+ * paragraph) shall be included in all copies or substantial portions of the
+ * Software.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
+ * THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
+ * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
+ * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
+ * DEALINGS IN THE SOFTWARE.
  */
 #define _GNU_SOURCE
 #include <config.h>
@@ -43,7 +44,7 @@
 
 #define clip(val_, min_, max_) min((max_), max((min_), (val_)))
 
-struct tools_options options;
+struct tools_context context;
 
 struct touch {
 	int active;
@@ -62,8 +63,8 @@ struct window {
 	int absx, absy;
 
 	/* scroll bar positions */
-	int vx, vy;
-	int hx, hy;
+	double vx, vy;
+	double hx, hy;
 
 	/* touch positions */
 	struct touch touches[32];
@@ -263,6 +264,7 @@ change_ptraccel(struct window *w, double amount)
 static void
 handle_event_device_notify(struct libinput_event *ev)
 {
+	struct tools_context *context;
 	struct libinput_device *dev = libinput_event_get_device(ev);
 	struct libinput *li;
 	struct window *w;
@@ -279,17 +281,12 @@ handle_event_device_notify(struct libinput_event *ev)
 	    libinput_device_get_name(dev),
 	    type);
 
-	if (libinput_device_config_tap_get_finger_count(dev) > 0) {
-		enum libinput_config_status status;
-		status = libinput_device_config_tap_set_enabled(dev,
-								LIBINPUT_CONFIG_TAP_ENABLED);
-		if (status != LIBINPUT_CONFIG_STATUS_SUCCESS)
-			error("%s: Failed to enable tapping\n",
-			      libinput_device_get_sysname(dev));
-	}
-
 	li = libinput_event_get_context(ev);
-	w = libinput_get_user_data(li);
+	context = libinput_get_user_data(li);
+	w = context->user_data;
+
+	tools_device_apply_config(libinput_event_get_device(ev),
+				  &context->options);
 
 	if (libinput_event_get_type(ev) == LIBINPUT_EVENT_DEVICE_ADDED) {
 		for (i = 0; i < ARRAY_LENGTH(w->devices); i++) {
@@ -369,7 +366,7 @@ handle_event_axis(struct libinput_event *ev, struct window *w)
 			LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL)) {
 		value = libinput_event_pointer_get_axis_value(p,
 				LIBINPUT_POINTER_AXIS_SCROLL_VERTICAL);
-		w->vy += (int)value;
+		w->vy += value;
 		w->vy = clip(w->vy, 0, w->height);
 	}
 
@@ -377,7 +374,7 @@ handle_event_axis(struct libinput_event *ev, struct window *w)
 			LIBINPUT_POINTER_AXIS_SCROLL_HORIZONTAL)) {
 		value = libinput_event_pointer_get_axis_value(p,
 				LIBINPUT_POINTER_AXIS_SCROLL_HORIZONTAL);
-		w->hx += (int)value;
+		w->hx += value;
 		w->hx = clip(w->hx, 0, w->width);
 	}
 }
@@ -435,7 +432,8 @@ static gboolean
 handle_event_libinput(GIOChannel *source, GIOCondition condition, gpointer data)
 {
 	struct libinput *li = data;
-	struct window *w = libinput_get_user_data(li);
+	struct tools_context *context = libinput_get_user_data(li);
+	struct window *w = context->user_data;
 	struct libinput_event *ev;
 
 	libinput_dispatch(li);
@@ -494,24 +492,6 @@ sockets_init(struct libinput *li)
 	g_io_add_watch(c, G_IO_IN, handle_event_libinput, li);
 }
 
-static int
-open_restricted(const char *path, int flags, void *user_data)
-{
-	int fd = open(path, flags);
-	return fd < 0 ? -errno : fd;
-}
-
-static void
-close_restricted(int fd, void *user_data)
-{
-	close(fd);
-}
-
-static const struct libinput_interface interface = {
-	.open_restricted = open_restricted,
-	.close_restricted = close_restricted,
-};
-
 int
 main(int argc, char *argv[])
 {
@@ -521,16 +501,17 @@ main(int argc, char *argv[])
 
 	gtk_init(&argc, &argv);
 
-	tools_init_options(&options);
+	tools_init_context(&context);
 
-	if (tools_parse_args(argc, argv, &options) != 0)
+	if (tools_parse_args(argc, argv, &context) != 0)
 		return 1;
 
 	udev = udev_new();
 	if (!udev)
 		error("Failed to initialize udev\n");
 
-	li = tools_open_backend(&options, &w, &interface);
+	context.user_data = &w;
+	li = tools_open_backend(&context);
 	if (!li)
 		return 1;
 
